/**
 * @file
 * Handle slick slider section functionality.
 */

(function (Drupal, $, once, drupalSettings) {
  /**
   * Get slider section id from element.
   *
   * @param obj sliderElem
   *   Slider page element.
   *
   * @return int
   *   Slider section id.
   */
  function getSliderSectionId(sliderElem) {
    let sliderSectionId = null;
    // Loop through element classes.
    sliderElem.classList.forEach(function (className) {
      // Check for class with id number.
      if (className.includes('slider-section-id-')) {
        // Get the integer value from the class string.
        const strId = className.replace(/[^\d.]/g, '');
        sliderSectionId = parseInt(strId, 10);
      }
    });
    return sliderSectionId;
  }

  /**
   * Shuffles order of child elements under parent element
   *
   * @param NodeList slides
   *   NodeList of slide elements.
   */
  function shuffleSlides(slides) {
    // Convert NodeList to array.
    const slidesArray = Array.prototype.slice.call(slides, 0);

    // Shuffle into random order.
    slidesArray.sort(function (a, b) {
      return Math.round(Math.random()) - 0.5;
    });

    // For each slide in shuffled order, detach and re-append to parent.
    slidesArray.forEach((childElem) => {
      const parent = childElem.parentElement;
      parent.removeChild(childElem);
      parent.appendChild(childElem);
    });
  }

  Drupal.behaviors.sliderLayoutsSlickInit = {
    attach(context) {
      // Get slider section elements.
      const $sliders = once('sliderSection', '.layout--slider-layout', context);
      $sliders.forEach(function (slide) {
        // Get id value, to get config from specific layout.
        const sliderSectionId = getSliderSectionId(slide);
        // Search for child element.
        const slider = slide.querySelector('.slider-main');

        // Default slick settings for all styles, can be overwritten.
        const slickSettings = {
          accessibility: true,
          slide: '.block:not(.layout-builder-block)',
        };

        // Check for slick settings passed in from layout config.
        if (
          'sliderSection' in drupalSettings &&
          typeof drupalSettings.sliderSection[sliderSectionId] !==
            'undefined' &&
          'slickSettings' in drupalSettings.sliderSection[sliderSectionId]
        ) {
          const config = drupalSettings.sliderSection[sliderSectionId];

          // Parse JSON string to convert to JS object.
          const layoutSliderStyle =
            config.slickSettings.length > 0
              ? JSON.parse(config.slickSettings)
              : null;

          // Merge with defaults, so settings passed in will override.
          if (layoutSliderStyle) {
            Object.assign(slickSettings, layoutSliderStyle);
          }

          if (config.shuffle) {
            // Get slides under parent.
            const slides = slider.querySelectorAll(
              '.block:not(.layout-builder-block)',
            );
            // Shuffle.
            shuffleSlides(slides);
          }
        }

        // Activate slick slider.
        $(slider).slick(slickSettings);
      });
    },
  };
})(Drupal, jQuery, once, drupalSettings);
