<?php

declare(strict_types=1);

namespace Drupal\slider_layouts\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\slider_layouts\SliderLayoutsService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;
use Drupal\Core\Cache\CacheTagsInvalidatorInterface;

/**
 * Configure Slick Layouts settings for this site.
 */
final class SliderSettingsForm extends ConfigFormBase {

  /**
   * The UUID service.
   *
   * @var \Drupal\Component\Uuid\UuidInterface
   */
  protected $uuidService;

  /**
   * The cache tags invalidator service.
   *
   * @var \Drupal\Core\Cache\CacheTagsInvalidatorInterface
   */
  protected $cacheTagsInvalidator;

  /**
   * The slider layouts service.
   *
   * @var \Drupal\slider_layouts\SliderLayoutsService
   */
  protected SliderLayoutsService $sliderLayoutsService;

  /**
   * Constructor for the SliderSettingsForm class.
   *
   * @param \Drupal\Component\Uuid\UuidInterface $uuid_service
   *   The UUID service.
   * @param \Drupal\Core\Cache\CacheTagsInvalidatorInterface $cache_tags_invalidator
   *   The cache tags invalidator service.
   * @param \Drupal\slider_layouts\SliderLayoutsService $slider_layouts_service
   *   The slider layouts service.
   */
  public function __construct(UuidInterface $uuid_service, CacheTagsInvalidatorInterface $cache_tags_invalidator, SliderLayoutsService $slider_layouts_service) {
    $this->uuidService = $uuid_service;
    $this->cacheTagsInvalidator = $cache_tags_invalidator;
    $this->sliderLayoutsService = $slider_layouts_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('uuid'),
      $container->get('cache_tags.invalidator'),
      $container->get('slider_layouts.service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'slider_layouts_slider_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['slider_layouts.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('slider_layouts.settings');

    // Get default slick settings from json file.
    $default_slick_settings = $this->sliderLayoutsService->getDefaultSlickSettings();

    $form['example'] = [
      '#type' => 'markup',
      '#markup' => '<div>Example:</div><pre>' . htmlspecialchars($default_slick_settings) . '</pre>',
    ];

    $form['styles'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Styles'),
      '#prefix' => '<div id="styles-wrapper">',
      '#suffix' => '</div>',
    ];

    // Retrieve styles from form state or configuration.
    if ($form_state->get('styles') === NULL) {
      $styles = $config->get('styles') ?: [];
      $form_state->set('styles', $styles);
    }
    else {
      $styles = $form_state->get('styles');

      // Update input with current style data.
      $input = $form_state->getUserInput();
      $input['styles'] = $styles;
      $form_state->setUserInput($input);
    }

    foreach ($styles as $key => $style) {
      $form['styles'][$key] = [
        '#type' => 'fieldset',
      ];
      // Name field.
      $form['styles'][$key]['name'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Name'),
        '#default_value' => $style['name'] ?? '',
        '#parents' => ['styles', $key, 'name'],
      ];
      // Settings field.
      $form['styles'][$key]['settings'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Settings'),
        '#default_value' => $style['settings'] ?? '',
        '#rows' => 10,
        '#parents' => ['styles', $key, 'settings'],
      ];

      // If the first element, add a reset button.
      if ($key == 'default') {
        $form['styles'][$key]['name']['#disabled'] = TRUE;

        $form['styles'][$key]['reset'] = [
          '#type' => 'submit',
          '#value' => $this->t('Reset'),
          '#submit' => ['::resetDefaultStyle'],
          '#ajax' => [
            'callback' => '::ajaxCallback',
            'wrapper' => 'styles-wrapper',
          ],
          '#name' => "reset_style_{$key}",
        ];
      }
      // Otherwise add a remove button.
      else {
        $form['styles'][$key]['remove'] = [
          '#type' => 'submit',
          '#value' => $this->t('Remove'),
          '#submit' => ['::removeStyle'],
          '#validate' => ['::validateRemoveStyle'],
          '#ajax' => [
            'callback' => '::ajaxCallback',
            'wrapper' => 'styles-wrapper',
          ],
          '#attributes' => [
            'class' => [
              'button--danger',
            ],
          ],
          '#name' => "remove_style_{$key}",
        ];
      }
    }

    // Add style button.
    $form['styles']['add_style'] = [
      '#type' => 'submit',
      '#value' => $this->t('+ Add Style'),
      '#submit' => ['::addStyle'],
      '#ajax' => [
        'callback' => '::ajaxCallback',
        'wrapper' => 'styles-wrapper',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * Adds a new style.
   */
  public function addStyle(array &$form, FormStateInterface $form_state): void {
    // Get styles from form state.
    $styles = $form_state->get('styles') ?: [];

    // Add default style if no styles exist.
    if (empty($styles)) {
      $styles['default'] = [
        'name' => 'Default',
        'settings' => $this->sliderLayoutsService->getDefaultSlickSettings(),
      ];
    }
    // Otherwise, add an empty style.
    else {
      $uuid = $this->uuidService->generate();
      $styles[$uuid] = [
        'name' => '',
        'settings' => '',
      ];
    }
    $form_state->set('styles', $styles);

    // Rebuild the form.
    $form_state->setRebuild(TRUE);
  }

  /**
   * Custom validation callback for the Remove button.
   */
  public function validateRemoveStyle(array &$form, FormStateInterface $form_state): void {
    // Get triggering element index number.
    $triggering_element = $form_state->getTriggeringElement();
    $name = $triggering_element['#name'];
    $style_id = str_replace('remove_style_', '', $name);

    // Load paths by style id, if the style is in use.
    $styles_in_use_paths = $this->sliderLayoutsService->getPathsForStyle($style_id);

    // If sections are found that use the style, display an error message.
    if (!empty($styles_in_use_paths)) {
      $links = [];
      // Create links from paths.
      foreach ($styles_in_use_paths as $path) {
        if (!empty($path)) {
          // Build a link from url path.
          $url = Url::fromUserInput($path, ['attributes' => ['target' => '_blank']]);
          $links[] = Link::fromTextAndUrl($path, $url)->toString();
        }
      }

      // If there are paths for sections that use the style,
      // display them in the error message.
      if (!empty($links)) {
        $links_list = '<ul><li>' . implode('</li><li>', $links) . '</li></ul>';
        $form_state->setErrorByName("styles][$style_id][remove", Markup::create("This style is in use and cannot be removed. Please check the following section configs:  {$links_list}"));
      }
      // If no paths are found, display a generic error message.
      else {
        $form_state->setErrorByName("styles][$style_id][remove", $this->t('This style is in use and cannot be removed.'));
      }
    }
  }

  /**
   * Removes a style.
   */
  public function removeStyle(array &$form, FormStateInterface $form_state): void {
    // Get triggering element index number.
    $triggering_element = $form_state->getTriggeringElement();
    $name = $triggering_element['#name'];
    $key = str_replace('remove_style_', '', $name);

    // Get styles from form state.
    $styles = $form_state->get('styles');

    // Remove triggering style.
    unset($styles[$key]);
    $form_state->set('styles', $styles);

    // Rebuild the form.
    $form_state->setRebuild(TRUE);
  }

  /**
   * Resets the Default style to the default configuration.
   */
  public function resetDefaultStyle(array &$form, FormStateInterface $form_state): void {
    // Get styles from form state.
    $styles = $form_state->get('styles') ?: [];

    // Set default values.
    $styles['default']['name'] = 'Default';
    $styles['default']['settings'] = $this->sliderLayoutsService->getDefaultSlickSettings();

    // Set styles.
    $form_state->set('styles', $styles);

    // Rebuild the form.
    $form_state->setRebuild(TRUE);
  }

  /**
   * Ajax callback to refresh the styles fieldset.
   */
  public function ajaxCallback(array &$form, FormStateInterface $form_state): array {
    return $form['styles'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    // Get styles from form state.
    $styles = $form_state->getValue('styles');

    // Loop through styles.
    foreach ($styles as $key => $style) {
      if (!empty($style['settings'])) {
        // Convert text to JSON.
        $settings = $this->sliderLayoutsService->convertTextToJson($style['settings']);
        $settings = json_decode($settings);

        // Through form error in json is invalid.
        if (!empty($settings) && $settings === NULL && json_last_error() !== JSON_ERROR_NONE) {
          $form_state->setErrorByName("styles][$key][settings", $this->t('The settings for style %name are not valid JSON.', ['%name' => $style['name']]));
        }
      }
    }
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    // Get styles from form state.
    $styles = $form_state->getValue('styles');

    // Set styles values in config.
    $this->config('slider_layouts.settings')
      ->set('styles', $styles)
      ->save();

    // Invalidate slider_layouts.config cache tags.
    $this->cacheTagsInvalidator->invalidateTags(['slider_layouts.config']);

    parent::submitForm($form, $form_state);
  }

}
