<?php

namespace Drupal\slider_layouts\Plugin\Layout;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Layout\LayoutDefault;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\slider_layouts\SliderLayoutsService;
use Drupal\Core\Path\CurrentPathStack;

/**
 * Slider Layout plugin.
 */
class SliderLayout extends LayoutDefault implements ContainerFactoryPluginInterface {

  /**
   * Static count value for how many slider sections there are on the page.
   *
   * @var int
   */
  protected static $sliderSectionCount = 0;

  /**
   * Unique id value for slider section.
   *
   * @var int
   */
  protected $sliderSectionId;

  /**
   * Config factory entity.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $sliderLayoutsConfig;

  /**
   * The slider layouts service.
   *
   * @var \Drupal\slider_layouts\SliderLayoutsService
   */
  protected SliderLayoutsService $sliderLayoutsService;

  /**
   * The current path service.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * Constructs a new instance of the plugin.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory service.
   * @param \Drupal\slider_layouts\SliderLayoutsService $slider_layouts_service
   *   The slider layouts service.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path service.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, ConfigFactoryInterface $config_factory, SliderLayoutsService $slider_layouts_service, CurrentPathStack $current_path) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->sliderLayoutsConfig = $config_factory->getEditable('slider_layouts.settings');
    $this->sliderLayoutsService = $slider_layouts_service;
    $this->currentPath = $current_path;

    // Set section id and increment static count.
    $this->sliderSectionId = self::$sliderSectionCount++;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
      $container->get('slider_layouts.service'),
      $container->get('path.current')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return parent::defaultConfiguration() + [
      // Default value for slick settings textarea.
      'slick_settings' => "",
      'slider_style' => 'default',
      'shuffle' => FALSE,
      'use_intro_text' => FALSE,
      'intro' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    // Load the slider_layouts.settings configuration.
    $slider_settings_styles = $this->sliderLayoutsConfig->get('styles');
    $style_options = array_map(function ($style): mixed {
      return $style['name'];
    }, $slider_settings_styles);

    // Checkbox whether or not to add intro text at the top of the section.
    $form['use_intro_text'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display text at the top of the section'),
      '#default_value' => $this->configuration['use_intro_text'] ?? FALSE,
    ];

    // Optional intro text field.
    $form['intro'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Intro'),
      '#format' => $this->configuration['intro']['format'] ?? 'full_html',
      '#default_value' => $this->configuration['intro']['value'] ?? NULL,
      '#states'   => [
        'visible' => [
          ':input[name="layout_settings[use_intro_text]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Choose a preset slider style.
    $form['slider_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Slider Style'),
      '#options' => $style_options,
      '#default_value' => $this->configuration['slider_style'] ?? 'default',
      '#description' => $this->t('Select the style for the slider. Configure styles at <a href=":url" target="_blank">Slider Layout settings</a>.', [':url' => '/admin/config/content/slider-layouts']),
      '#empty_option' => $this->t('--Use custom style in advanced settings--'),
      '#empty_value' => 'none',
    ];

    // Advanced config section.
    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced'),
      '#open' => TRUE,
      '#states'   => [
        'visible' => [
          ':input[name="layout_settings[slider_style]"]' => ['value' => 'none'],
        ],
      ],
    ];

    // Slick settings textarea form element.
    $form['advanced']['slick_settings'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom Style'),
      '#default_value' => $this->configuration['slick_settings'],
      '#description' => $this->t('Settings to pass into slick slider, structured like a JavaScript Object. Settings details can be found here <a href="https://github.com/kenwheeler/slick">https://github.com/kenwheeler/slick</a>.'),
      '#rows' => 10,
    ];

    // Shuffle checkbox form element.
    $form['shuffle'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Shuffle?'),
      '#default_value' => $this->configuration['shuffle'],
      '#description' => $this->t('Whether or not to shuffle slide order.'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    // Advanced config section.
    $advanced_config = $form_state->getValue('advanced');
    $custom_style = $advanced_config['slick_settings'];
    $slider_style = $form_state->getValue('slider_style');

    // Make sure there is either a style or custom style entered.
    if ((!isset($slider_style) || $slider_style === 'none') && empty($custom_style)) {
      $form_state->setErrorByName('slider_style', $this->t('Please select a slider style or enter a custom style in advanced settings.'));
    }

    if (!empty($custom_style)) {
      // Convert text to JSON.
      $slick_settings = $this->sliderLayoutsService->convertTextToJson($custom_style);
      $slick_settings = json_decode($slick_settings);

      // Check if JSON is valid, throw error if not.
      if (is_null($slick_settings)) {
        $form_state->setErrorByName('slick_settings', $this->t('The Custom Style field has invalid structure. Please use valid JSON.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    // Advanced config section.
    $advanced_config = $form_state->getValue('advanced');

    // Save config values.
    $this->configuration['intro'] = $form_state->getValue('intro');
    $this->configuration['use_intro_text'] = $form_state->getValue('use_intro_text');
    $this->configuration['slider_style'] = $form_state->getValue('slider_style');
    $this->configuration['shuffle'] = $form_state->getValue('shuffle');
    $this->configuration['slick_settings'] = $advanced_config['slick_settings'];

    // Keep track of which styles are being used in module config.
    if ($this->configuration['slider_style'] !== 'none') {
      // Mark style as in use.
      $this->sliderLayoutsService->markStyleInUse($this->currentPath->getPath(), $this->configuration['slider_style']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build(array $regions) {
    $build = parent::build($regions);

    // Load styles from slider layouts config.
    $slider_settings_styles = $this->sliderLayoutsConfig->get('styles');
    // Get slider style index, if a valid.
    $slider_style_index = isset($this->configuration['slider_style']) && ($this->configuration['slider_style'] == 'none' || (is_array($slider_settings_styles) && !empty($slider_settings_styles[$this->configuration['slider_style']]))) ? $this->configuration['slider_style'] : 'default';

    // Render the text_format field.
    if ($this->configuration['use_intro_text'] && !empty($this->configuration['intro']['value'])) {
      $build['intro'] = [
        '#type' => 'processed_text',
        '#text' => $this->configuration['intro']['value'],
        '#format' => $this->configuration['intro']['format'],
      ];
    }

    // Add a class with slider section id.
    $build['#attributes']['class'] = [
      "slider-section-id-{$this->sliderSectionId}",
    ];

    if (isset($slider_style_index)) {
      // Use slick settings from Style.
      if ($slider_style_index !== 'none' && !empty($slider_settings_styles[$slider_style_index]['settings'])) {
        $slick_settings = $this->sliderLayoutsService->convertTextToJson($slider_settings_styles[$slider_style_index]['settings']);
      }
      // Use advanced custom slick settings.
      elseif ($slider_style_index === 'none' && !empty($this->configuration['slick_settings'])) {
        $slick_settings = $this->sliderLayoutsService->convertTextToJson($this->configuration['slick_settings']);
      }
    }

    // If slick settings are still empty, set to default.
    if (empty($slick_settings)) {
      $slick_settings = $this->sliderLayoutsService->convertTextToJson($this->sliderLayoutsService->getDefaultSlickSettings());
    }

    // Pass config to JavaScript.
    $build['#attached']['drupalSettings']['sliderSection'][$this->sliderSectionId] = [
      'slickSettings' => !empty($slick_settings) ? $slick_settings : '',
      'shuffle' => $this->configuration['shuffle'] ?? FALSE,
    ];

    // Add cache tag.
    $build['#cache']['tags'][] = 'slider_layouts.config';

    return $build;
  }

}
