<?php

namespace Drupal\slider_layouts;

use Drupal\Core\Extension\ModuleExtensionList;
use Drupal\Core\Database\Connection;

/**
 * Class SliderLayoutsService.
 *
 * The SliderLayoutsService class provides functionality
 * to manage and manipulate slider layouts.
 */
class SliderLayoutsService {

  /**
   * The module extension list service.
   *
   * @var \Drupal\Core\Extension\ModuleExtensionList
   */
  protected ModuleExtensionList $moduleExtensionList;

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $connection;

  /**
   * Constructor for the SliderSettingsForm class.
   *
   * @param \Drupal\Core\Extension\ModuleExtensionList $module_extension_list
   *   The module extension list service.
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   */
  public function __construct(ModuleExtensionList $module_extension_list, Connection $connection) {
    $this->moduleExtensionList = $module_extension_list;
    $this->connection = $connection;
  }

  /**
   * Retrieves the default settings for the Slick carousel.
   *
   * This function returns an array of default configuration settings
   * for the Slick carousel. These settings can be used to initialize
   * the carousel with predefined options.
   *
   * @return string
   *   An associative array containing the default settings
   *   for the Slick carousel.
   */
  public function getDefaultSlickSettings(): string {
    $module_path = $this->moduleExtensionList->getPath('slider_layouts');
    $json_path = "{$module_path}/js/default_slick_settings.json";

    // Load json file for default settings.
    if (file_exists($json_path)) {
      $json_data = file_get_contents($json_path);
      // Remove quotes from property keys.
      return preg_replace('/([\'"])?([a-z0-9A-Z_]+)([\'"])?:/', '$2:', $json_data);
    }

    return '';
  }

  /**
   * Converts text from a textarea field to JSON format.
   *
   * @param string $text
   *   Text string from textarea form element.
   *
   * @return string
   *   Text converted to JSON format.
   */
  public function convertTextToJson($text): array|string|null {
    if (empty($text)) {
      return '';
    }

    // Remove whitespace.
    $text = preg_replace('/\s+/', '', $text);
    // Add quotes to property name if it doesn't exist.
    return preg_replace('/([\'"])?([a-z0-9A-Z_]+)([\'"])?:/', '"$2": ', $text);
  }

  /**
   * Sets or updates a style for a given path.
   *
   * @param string $path
   *   The relative path.
   * @param string $style_id
   *   The UUID of the style.
   */
  public function markStyleInUse(string $path, string $style_id): void {
    // Avoid paths ending in /slider_layout,
    // as they are not the paths we need.
    if (!str_ends_with($path, '/slider_layout')) {
      $this->connection->merge('styles_in_use')
        ->key(['path' => $path])
        ->fields(['style_id' => $style_id])
        ->execute();
    }
  }

  /**
   * Gets all paths using a given style UUID.
   *
   * @param string $style_id
   *   The UUID of the style.
   *
   * @return string[]
   *   An array of paths.
   */
  public function getPathsForStyle(string $style_id): array {
    return $this->connection->select('styles_in_use', 's')
      ->fields('s', ['path'])
      ->condition('style_id', $style_id)
      ->execute()
      ->fetchCol();
  }

}
